
# q-expansion of newform 2736.3.o.h, downloaded from the LMFDB on 07 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 2736
weight = 3
poly_data = [2, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0], 1], [[0, 6], 1]]

hecke_ring_character_values = [[1711, [1, 0]], [2053, [1, 0]], [1217, [1, 0]], [1009, [-1, 0]]]
aps_data = [[0, 0], [0, 0], [1, 0], [-5, 0], [5, 0], [0, -2], [25, 0], [-19, 0], [-10, 0], [0, -5], [0, 5], [0, -3], [0, 5], [-5, 0], [5, 0], [0, 3], [0, 10], [95, 0], [0, -13], [0, 0], [-25, 0], [0, -5], [-130, 0], [0, 15], [0, 2], [-50, 0], [0, 2], [0, -12], [0, 15], [0, -13], [0, -27], [-163, 0], [-95, 0], [-125, 0], [-215, 0], [0, 10], [-190, 0], [-110, 0], [0, 7], [0, 22], [0, -15], [0, -30], [293, 0], [0, 7], [70, 0], [-173, 0], [0, -10], [0, 43], [0, 8], [-145, 0], [-335, 0], [197, 0], [0, 35], [173, 0], [0, 8], [-355, 0], [0, 45], [-110, 0], [-265, 0], [0, -50], [-125, 0], [0, -22], [0, -33], [-235, 0], [-310, 0], [0, 22], [0, -35], [0, -62], [125, 0], [23, 0], [-410, 0], [-475, 0], [-230, 0], [0, -8], [0, 30], [0, -17], [553, 0], [335, 0], [0, -25], [0, 85], [62, 0], [0, 35], [0, -60], [0, -27], [0, -95], [365, 0], [0, 90], [-265, 0], [553, 0], [-485, 0], [-115, 0], [-490, 0], [0, 72], [-82, 0], [-485, 0], [-250, 0], [0, 20], [0, 15], [0, -42], [-25, 0], [0, -2], [745, 0], [0, 37], [0, -50], [-1070, 0], [-25, 0], [725, 0], [-650, 0], [0, -35], [0, -100], [0, 32], [1055, 0], [505, 0], [130, 0], [475, 0], [0, 100], [955, 0], [965, 0], [-935, 0], [0, -10], [0, -80], [0, -22], [0, -107], [0, 132], [715, 0], [430, 0], [-382, 0], [-115, 0], [1075, 0], [530, 0], [547, 0], [0, -113], [0, 20], [1055, 0], [-215, 0], [-145, 0], [0, -48], [0, -22], [0, 83], [457, 0], [0, -60], [-167, 0], [1315, 0], [0, -63], [0, 90], [0, -40], [770, 0], [0, 148], [-557, 0], [0, 117], [0, 22], [25, 0], [-965, 0], [0, -92], [0, 37], [0, 110], [538, 0], [742, 0], [335, 0], [0, 50], [-1210, 0], [0, -117], [-350, 0], [0, 30], [0, 47], [0, -82], [0, -45], [-265, 0], [0, 75], [-215, 0], [0, -145], [0, -5], [-730, 0], [-1030, 0], [0, -190], [-575, 0], [-302, 0], [1465, 0], [0, 78], [1535, 0], [355, 0], [0, 80], [0, -62], [0, -138], [485, 0], [193, 0], [0, -102], [0, -2], [0, -125], [398, 0], [0, -213], [2165, 0], [0, 105], [0, -65], [-2050, 0], [0, 212], [-577, 0], [-670, 0], [-410, 0], [1685, 0], [0, -70], [0, 265], [0, 43], [0, 215], [-1330, 0], [1105, 0], [1555, 0], [0, 238], [-1103, 0], [0, -160], [650, 0], [190, 0], [-365, 0], [0, -88], [0, -35], [0, -25], [-1070, 0], [0, -80], [0, -33], [1225, 0], [0, -140], [0, 215], [0, 275], [1915, 0], [0, 148], [-1417, 0], [0, -38], [0, 75], [0, -308], [-427, 0], [-145, 0], [0, -285], [0, -145], [0, 20], [715, 0], [-355, 0], [-505, 0], [-1055, 0], [1790, 0], [0, -195], [0, 308], [2563, 0], [-2150, 0], [0, 120], [0, 143], [-1507, 0], [1915, 0], [0, 55], [0, 200], [605, 0], [-1630, 0], [-2927, 0], [1790, 0], [0, -115], [1345, 0], [1838, 0], [983, 0], [0, -173], [0, -177], [-790, 0], [0, -143], [0, -298], [-1585, 0], [0, -348], [-2375, 0], [0, 175], [0, -165], [-1895, 0], [0, 92], [-1655, 0], [0, 235], [0, -207], [-1705, 0], [-2285, 0], [0, -23], [-1190, 0], [-595, 0], [0, -385], [0, -210], [-715, 0], [0, 268], [-3590, 0], [-2155, 0], [0, 140], [-1205, 0], [-3355, 0], [-1465, 0], [0, -268], [283, 0], [0, 120], [-1202, 0], [245, 0], [0, -382], [0, 220], [0, -172], [1270, 0], [0, 310], [-1535, 0], [0, 135], [-5, 0], [-2665, 0], [0, 42], [2635, 0]]
